/****************************************
 *   MP006
 *   LCD tester 
 *   dsPIC30F3013
 ****************************************/

#include <p30Fxxxx.h>
#include <stdio.h>
#include <dsp.h>
#include <adc12.h>
#include <outcompare.h>
#include <timer.h>

		_FWDT(WDT_OFF);
		_FGS(CODE_PROT_OFF);
		_FOSC(CSW_FSCM_OFF & FRC_PLL16);
		//_FOSC(CSW_FSCM_OFF & FRC_PLL8);
		//_FOSC(CSW_FSCM_OFF & FRC_PLL4);
		_FBORPOR(PBOR_OFF & PWRT_64 & MCLR_EN);
		
// Prototype
void lcd_init(void);
unsigned char get_value(int hs, int vs);
unsigned char get_title_osci(int hs, int vs);
unsigned char get_title_logi(int hs, int vs);
unsigned char get_font_data(char c, int h_pos, int v_pos);
unsigned char get_pict1(int hs, int vs);
unsigned char get_pict2(int hs, int vs);

void wait( unsigned int count);
unsigned char oscilo_disp(int hs, int vs);
unsigned char logiana_disp(int hs, int vs);
unsigned char tester_disp(int hs, int vs);

unsigned char get_pict(int hs, int vs);


//   Port definition
#define RED0_TRIS			(TRISBbits.TRISB0)	// Ref RED0
#define RED0_IO				(PORTBbits.RB0)	
#define RED1_TRIS			(TRISBbits.TRISB1)	// Ref RED1
#define RED1_IO				(PORTBbits.RB1)
	
#define GREEN0_TRIS			(TRISBbits.TRISB2)	// Ref GREEN0
#define GREEN0_IO			(PORTBbits.RB2)	
#define GREEN1_TRIS			(TRISBbits.TRISB3)	// Ref GREEN1
#define GREEN1_IO			(PORTBbits.RB3)

#define BLUE0_TRIS			(TRISBbits.TRISB4)	// Ref BLUE0
#define BLUE0_IO			(PORTBbits.RB4)	
#define BLUE1_TRIS			(TRISBbits.TRISB5)	// Ref BLUE1
#define BLUE1_IO			(PORTBbits.RB5)
	
#define HS_TRIS				(TRISCbits.TRISC15)	// Ref HS
#define HS_IO				(PORTCbits.RC15)	
#define VS_TRIS				(TRISCbits.TRISC13)	// Ref VS
#define VS_IO				(PORTCbits.RC13)
#define CLK_TRIS			(TRISCbits.TRISC14)	// Ref CLK
#define CLK_IO				(PORTCbits.RC14)	

#define DE_TRIS				(TRISDbits.TRISD9)	// Ref DE
#define DE_IO				(PORTDbits.RD9)	

//#define OHM0_TRIS			(TRISDbits.TRISD9)	// Ref HS
//#define OHM0_IO				(PORTDbits.RD9)	
#define OHM1_TRIS			(TRISDbits.TRISD8)	// Ref VS
#define OHM1_IO				(PORTDbits.RD8)
#define OHM2_TRIS			(TRISBbits.TRISB6)	// Ref CLK
#define OHM2_IO				(PORTBbits.RB6)	

#define OC1_TRIS			(TRISBbits.TRISB8)	// For -9V
#define OC1_IO				(PORTBbits.RB8)	
#define OC2_TRIS			(TRISBbits.TRISB9)	// For +9V
#define OC2_IO				(PORTBbits.RB9)	

// Input
#define TERM0_TRIS			(TRISBbits.TRISB7)	// Ref TERM0
#define TERM0_IO			(PORTBbits.RB7)	
#define TERM1_TRIS			(TRISFbits.TRISF4)	// Ref TERM1
#define TERM1_IO			(PORTFbits.RF4)	
#define TERM2_TRIS			(TRISFbits.TRISF5)	// Ref TERM2
#define TERM2_IO			(PORTFbits.RF5)	
#define TERM3_TRIS			(TRISFbits.TRISF6)	// Ref TERM3
#define TERM3_IO			(PORTFbits.RF6)	

#define ON	(1)
#define OFF	(0)

#define H_WIDTH		(410)
#define TITLE_WIDTH_NORM (80)
#define TITLE_WIDTH_TEST (160)

#define FONT_TEST	(0)
#define OSCILO		(1)
#define LOGIANA		(2)
#define TESTER		(3)

//=================================================
// Color Defnition
//=================================================
#define WHITE			(0)
#define BLUE			(1)
#define RED				(2)
#define DARK_BLUE		(3)
#define YELLOW			(4)
#define GREEN			(5)
#define RED2			(6)
#define BLACK			(7)

unsigned char signal_color     =BLACK;
unsigned char back_ground_color=GREEN;

// A/D control register
unsigned int _ADCON1=ADC_MODULE_ON &  /* A/D Converter on */
			ADC_IDLE_CONTINUE &       /* A/D Operate in Idle mode */
			ADC_FORMAT_INTG &         /* A/D data format integer */
			ADC_CLK_TMR &             /* Timer compare ends sampling and starts conversion */
			ADC_CLK_AUTO & 
            ADC_AUTO_SAMPLING_ON &    /* Auto sampling Select */
			ADC_SAMP_OFF;             /* sample / hold amplifiers are holding */
unsigned int _ADCON2=ADC_VREF_AVDD_AVSS & /* A/D Voltage reference configuration Vref+ is AVdd and Vref- is AVss */
			ADC_SCAN_OFF &            /* A/D Do notScan Input Selections for CH0+ during SAMPLE A */
            ADC_SAMPLES_PER_INT_1 &   /* A/D Interrupt at the completion of conversion for each sample */
			ADC_ALT_BUF_OFF &         /* Buffer configured as 1 16-word buffer */
			ADC_ALT_INPUT_OFF;        /* use MUXA only */

			//unsigned int _ADCON3=ADC_SAMPLE_TIME_12 & /* A/D Auto Sample Time 12 Tad */
unsigned int _ADCON3=ADC_SAMPLE_TIME_20 & /* A/D Auto Sample Time 12 Tad */
			ADC_CONV_CLK_SYSTEM &     /* A/D Conversion Clock Source Clock derived from system clock */
			ADC_CONV_CLK_10Tcy;       /* A/D conversion clock select bit ADCS<5:0>*/
unsigned int _ADCHS = ADC_CH0_POS_SAMPLEA_AN3 & /* A/D Chan 0 pos i/p sel for SAMPLE A is AN3 */
			ADC_CH0_NEG_SAMPLEA_NVREF;	/* A/D CHA, CHB, CHC negative input is VREF- (0xFF7F) */
unsigned int _ADPCFG=ENABLE_AN7_ANA;	/*Enable AN7 in analog mode */
	
unsigned int _ADCSSL=SCAN_NONE;		/*Skip AN0-AN15 for Input Scan */

// PWM related setting
#define OCPWM_FCY	(7370000*8)/4   //7.37MHz 8PLL
#define OCPWM_FPWM	28800
#define OCPWM_FSAMP	8000
#define OCPWM_FPWM_FSAMP_RATIO	(OCPWM_FPWM/OCPWM_FSAMP)

//
#define OCPWM_HIGHEST_INPUT_VALUE	 4096
#define OCPWM_LOWEST_INPUT_VALUE	-4096

#define OCPWM_INPUT_RANGE			(OCPWM_HIGHEST_INPUT_VALUE-(OCPWM_LOWEST_INPUT_VALUE))
#define OCPWM_PWM_FACTOR			(OCPWM_MAX_PWM_PERIOD/OCPWM_INPUT_RANGE)

#define OCPWM_TMRPRESCALE			1
#define OCPWM_MAX_PWM_PERIOD		(((OCPWM_FCY/OCPWM_FPWM)*OCPWM_TMRPRESCALE)-1)


// PWM for output control register
unsigned int _OC1CON = OC_IDLE_CON& /* continue operation in idle mode */
        			   OC_TIMER2_SRC&   /* Timer2 is the clock source for OutputCompare1 */
					   OC_PWM_FAULT_PIN_DISABLE; /* PWM Mode on OCx, fault pin disabled */
unsigned int _OC1RS = ((OCPWM_MAX_PWM_PERIOD)/2);
unsigned int _OC1R  = ((OCPWM_MAX_PWM_PERIOD)/2);

unsigned int _OC2CON = OC_IDLE_CON& /* continue operation in idle mode */
        			   OC_TIMER2_SRC&   /* Timer2 is the clock source for OutputCompare1 */
					   OC_PWM_FAULT_PIN_DISABLE; /* PWM Mode on OCx, fault pin disabled */
unsigned int _OC2RS = ((OCPWM_MAX_PWM_PERIOD)/2);
unsigned int _OC2R  = ((OCPWM_MAX_PWM_PERIOD)/2);

// Data definition



unsigned int	data_area[160];
char     		disp_data[20];
unsigned char	sampling_flag=OFF;   // ON:Sampling OFF:Stop sampling

// Timer 3 Interrupt
void __attribute__((__interrupt__, __shadow__, no_auto_psv)) _AltT3Interrupt(void)
{
	// Application data
    unsigned int AD_data;
    static unsigned int pos = 0;
    static unsigned int save=0;
    
	while(!IFS0bits.ADIF);	// Wait A/D convert

	if( sampling_flag==ON){
		AD_data  = ReadADC12(7);
		AD_data  &= 0xFFF;
		data_area[pos] = AD_data;
		//data_area[pos] = pos;
		//data_area[pos] = (unsigned char)(PORTB&0x00FF);
		pos ++;
		if( pos> 160){
			pos=0;
			sampling_flag=OFF;
		}	
	}	
	
	//IFS0bits.ADIF = 0;	// Clear A/D Interrupt Flag
	IFS0bits.T3IF = 0;	// Clear Timer3 Flag
}
	
void lcd_init()  {
	
	// Use Alternate Interrupt Vector Table
	INTCON2bits.ALTIVT=1;
	
	RED0_TRIS = 0;
	RED1_TRIS = 0;
	GREEN0_TRIS = 0;
	GREEN1_TRIS = 0;
	BLUE0_TRIS = 0;
	BLUE1_TRIS = 0;
	
	HS_TRIS    = 0;
	VS_TRIS    = 0;
	CLK_TRIS   = 0;
	DE_TRIS = 0;
	
//	OHM0_TRIS = 0;
	OHM1_TRIS = 0;
	OHM2_TRIS = 0;
	
	OC1_TRIS  = 0;
	OC2_TRIS  = 0;
	
	//TRISB = 0x0000;

	// INPUT
	TERM0_TRIS = 1;
	TERM1_TRIS = 1;
	TERM2_TRIS = 1;
	TERM3_TRIS = 1;
		
//=================================================
//
//=================================================
	/*
	PORTB &= 0xBF;
	PORTB &= 0xEF;
	PORTB &= 0xDF;
	PORTB |= 0x80;
	*/
		
	// A/D Initialize
	SetChanADC12(_ADCHS);

	// Tad={Tcy(ADCS+1)}/2>334ns, Then ADCS>18.7, Tad=10*Tcy
	OpenADC12(_ADCON1, _ADCON2, _ADCON3, _ADPCFG, _ADCSSL);
		
	// Timer3 setting
	OpenTimer3(T3_ON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT, 1842-1);	// 8kHz sampling ADC
	ConfigIntTimer3(T3_INT_PRIOR_5 & T3_INT_ON);
		
	// Timer2 setting
	// Initialize Timer2 28KHz = Fcy /1024)
	OpenTimer2( 0xE020, 0x3FFF);	// 28kHz carrier PWM

    // PWM by output compair
	OpenOC1( _OC1CON, _OC1RS, 0x3FFF );
	OpenOC2( _OC2CON, _OC2RS, 0x3FFF );
	
	// Secondary Duty Cycle register(OC1RS)
	// base=1024/2=512 +-512 -> 0x3FF=10bit resolution
	SetDCOC1PWM(0x1FFF);	// -9V Set Data to OC PWM
	SetDCOC2PWM(0x1FFF);	// +9V Set Data to OC PWM
	
	DE_IO  = OFF;
	
	PORTB = 0x0000;

	CLK_IO = OFF;
	wait(1);
	VS_IO  = OFF;
	wait(1);
	HS_IO  = OFF;
	wait(1);
	
}


void wait( unsigned int count){
	static unsigned int i;

	return;
	for(i=0; i< count*10 ; i++);	
}	
	

int main() {

int 		COUNTER_VS = 0;
int 		COUNTER_HS = 0;
int			h_pos = TITLE_WIDTH_NORM;
float		value;
char		messege[100];
unsigned int prev_value;
int			timing=0, start_pos=0,m_length;
float		max=5.0, min=0.0, ave=2.5;
		
unsigned int  RGB_VALUE = 0;
unsigned char ope_flag  = TESTER;
//static	unsigned char ope_flag  = OSCILO;
//static	unsigned char ope_flag  = LOGIANA;
//static	unsigned char ope_flag  = FONT_TEST;
	
	
	lcd_init();
	
	//while( 1 )DE_IO = OFF;;
	
	switch(ope_flag){
		case FONT_TEST:
		case OSCILO:
		case LOGIANA:
			h_pos = TITLE_WIDTH_NORM;
			break;
		case TESTER:
			h_pos = TITLE_WIDTH_TEST;
			break;
		default:
			break;
	}
	memset(messege, 0x20, 100);
	
	while( 1 ){
  		
		sampling_flag = ON;
		
		if(ope_flag == TESTER){
			if(prev_value!=data_area[0]){
				value = (float)data_area[0];
				value = value/1024.0;
				sprintf(disp_data,"   %5.2f V  ", value);
			}
			timing++;
			if( timing >1){
				start_pos++;
				if(messege[start_pos]=='\0') start_pos=0;
				timing = 0;
				sprintf(messege,"Schmart Board 2010 MCU Challenge Max=%5.2fV Min=%5.2fV Average=%5.2fV  ", max,min,ave);
				m_length = strlen( messege);
			}
		}	
		DE_IO = OFF;
		
		// VS
		VS_IO = ON;
		wait(1);
   		for(COUNTER_VS = 0; COUNTER_VS <153 ;COUNTER_VS++){
			// CK
			CLK_IO = ON;
			wait(1);
			CLK_IO = OFF;
			wait(1);
   		}
   		VS_IO = OFF;
   		wait(1);

		// HS
		for(COUNTER_VS = 0; COUNTER_VS < 16 ;COUNTER_VS++){
			for(COUNTER_HS = 0; COUNTER_HS < 323 ;COUNTER_HS++){
				// CK
				CLK_IO = ON;
 				wait(1);
 				CLK_IO = OFF;
 				wait(1);
   			}
   			CLK_IO = ON;
   			wait(1);
   			HS_IO = ON;
   			wait(1);
   			
   			CLK_IO = OFF;
   			wait(1);
   			HS_IO = OFF;
   			wait(1);
  		} 		
		
		wait(1);
		
		//sampling_flag = OFF;
		
		for(COUNTER_VS=0; COUNTER_VS< 96; COUNTER_VS++){
 			for(COUNTER_HS = 0; COUNTER_HS< 107 ;COUNTER_HS++)
   			{
				// CK
				CLK_IO = ON;
				wait(1);
				CLK_IO = OFF;
				wait(1);
   			}
   			// Headder
   			for(COUNTER_HS = 0; COUNTER_HS< h_pos ;COUNTER_HS++)
   			{
				switch(ope_flag){
					case FONT_TEST:
						RGB_VALUE = (unsigned int)get_value( COUNTER_HS/2, COUNTER_VS/4);
						break;
					case OSCILO:
						RGB_VALUE = get_title_osci(COUNTER_HS, COUNTER_VS);
						break;
					case LOGIANA:
						RGB_VALUE = get_title_logi(COUNTER_HS, COUNTER_VS);
						break;
					case TESTER:
						if(COUNTER_VS<72){
							RGB_VALUE = tester_disp(COUNTER_HS/2, COUNTER_VS/4);
						}else{
							RGB_VALUE = get_font_data(messege[(COUNTER_HS/8+start_pos)%m_length], COUNTER_HS%8, COUNTER_VS/3);
						}
						if(COUNTER_VS==72){
							RGB_VALUE = 0x0C;
						}
						break;
					default:
						break;
				}		
				RGB_VALUE = RGB_VALUE&0x003F;

				PORTB &= 0xFFC0;
				wait(1);
				PORTB |= RGB_VALUE;
				wait(1);
				
				CLK_IO = ON;
				wait(1);
				CLK_IO = OFF;
				wait(1);
   			}
   			// Body
			for(COUNTER_HS = 0; COUNTER_HS < (H_WIDTH-h_pos) ;COUNTER_HS++)
   			{
				//COLOR SET
				switch(ope_flag){
					case FONT_TEST:
						RGB_VALUE = COUNTER_HS/2;
						RGB_VALUE = COUNTER_VS/8;
						//wait(1);
						break;
					case OSCILO:
						RGB_VALUE = oscilo_disp(COUNTER_HS/2, COUNTER_VS);
						break;
					case LOGIANA:
						RGB_VALUE = logiana_disp(COUNTER_HS/2, COUNTER_VS);
						break;
					case TESTER:
						if(COUNTER_VS<72){
							if(start_pos%2){
								RGB_VALUE = get_pict1((COUNTER_HS/2)%50, COUNTER_VS/2);
							}else{
								RGB_VALUE = get_pict2((COUNTER_HS/2)%50, COUNTER_VS/2);
							}	
						}else{
							RGB_VALUE = get_font_data(messege[((COUNTER_HS+h_pos)/8+start_pos)%m_length], COUNTER_HS%8, COUNTER_VS/3);
						}
						if(COUNTER_VS==72){
							RGB_VALUE = 0x0C;
						}	
						break;
					default:
						break;
				}	
				RGB_VALUE = RGB_VALUE&0x003F;

				PORTB &= 0xFFC0;
				wait(1);
				PORTB |= RGB_VALUE;
				
				// CK
				CLK_IO = ON;
				wait(1);
				CLK_IO = OFF;
				wait(1);
   			}
   			
   			CLK_IO = ON;
   			wait(1);
			HS_IO = ON;
			wait(1);
			
 			CLK_IO = OFF;
 			wait(1);
 			HS_IO = OFF;
 			wait(1);
  		}
	}
}

////////////////////////////////////////
unsigned char oscilo_disp(int hs, int vs)
{
	int max_value, min_value;
	int middle_value;
	
	middle_value = (data_area[hs]+data_area[hs+1])/2;

	if(hs%2 == 0){
		if( middle_value > data_area[hs]){
			max_value = middle_value;
			min_value= data_area[hs];
		}else{
			max_value= data_area[hs];
			min_value= middle_value;
		}
	}else{
		if( middle_value > data_area[hs+1]){
			max_value = middle_value;
			min_value= data_area[hs+1];
		}else{
			max_value= data_area[hs+1];
			min_value= middle_value;
		}
	}
	if(min_value<=vs && vs<=max_value)
	{
		return signal_color;
	}
	return back_ground_color;
}	
unsigned char logiana_disp(int hs, int vs)
{
	unsigned char value;
	
	value = data_area[hs];
	
	switch(vs/0x10){
	case (0):    // Upper Time pattern
	case (5):    // Lower Time pattern
		if( (hs%2)==1 ){
			return signal_color;
		}
		break;
	case (1):    // Port0
		if( value&0x01){
			return signal_color;
		}
		break;
	case (2):    // Port1
		if( value&0x02){
			return signal_color;
		}
		break;
	case (3):    // Port2
		if( value&0x04){
			return signal_color;
		}
		break;
	case (4):    // Port3
		if( value&0x08){
			return signal_color;
		}
		break;
	default:
		break;
	}	
				
	return back_ground_color;
}

unsigned char tester_disp(int hs, int vs)
{
	return get_font_data(disp_data[hs/8+2],hs%8, vs/2);
}
